import os
import sys
from pathlib import Path

from _pytest._py.path import LocalPath

from tests.common import COMPRESSION_DICT, PYTHON_EXE, SAMPLE_DIR, SRC_DIR, run_cmd

os.chdir(Path(__file__).resolve().parent)
sys.path.append("../src")

from sticker_convert.utils.media.codec_info import CodecInfo  # type: ignore # noqa: E402

SIZE_MAX_IMG = COMPRESSION_DICT.get("custom").get("size_max").get("img")
SIZE_MAX_VID = COMPRESSION_DICT.get("custom").get("size_max").get("vid")


def _run_sticker_convert(fmt: str, tmp_path: LocalPath) -> None:
    run_cmd(
        [
            PYTHON_EXE,
            "sticker-convert.py",
            "--input-dir",
            str(SAMPLE_DIR),
            "--output-dir",
            str(tmp_path),
            "--preset",
            "custom",
            "--duration-max",
            "2000",
            "--steps",
            "6",
            "--no-confirm",
            "--img-format",
            fmt,
            "--vid-format",
            fmt,
        ],
        cwd=SRC_DIR,
    )

    for i in SAMPLE_DIR.iterdir():
        preset_dict = COMPRESSION_DICT.get("custom")
        if i.name.startswith("static_") and preset_dict.get("fake_vid") is False:
            size_max = preset_dict.get("size_max").get("img")
        else:
            size_max = preset_dict.get("size_max").get("vid")

        fname = Path(i).stem + fmt
        fpath = Path(tmp_path) / fname
        fps, frames, duration = CodecInfo.get_file_fps_frames_duration(fpath)

        print(f"[TEST] Check if {fname} exists")
        assert fpath.is_file()
        assert os.path.getsize(fpath) < size_max

        if i.name.startswith("animated_"):
            print(f"[TEST] {fname}: {fps=} {frames=} {duration=}")
            duration_min = preset_dict.get("duration").get("min")
            duration_max = preset_dict.get("duration").get("max")
            assert fps <= preset_dict.get("fps").get("max")
            if duration_min:
                assert duration >= duration_min
            if duration_max:
                assert duration <= duration_max


def test_to_static_png(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".png", tmp_path)


def test_to_static_webp(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".webp", tmp_path)


def test_to_animated_apng(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".apng", tmp_path)


def test_to_animated_gif(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".gif", tmp_path)


def test_to_animated_webm(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".webm", tmp_path)


def test_to_animated_webp(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".webp", tmp_path)


def test_to_animated_mp4(tmp_path: LocalPath) -> None:
    _run_sticker_convert(".mp4", tmp_path)



#!/usr/bin/env python3
from typing import TYPE_CHECKING, Any, Optional

from tqdm import tqdm
from ttkbootstrap import LabelFrame, Progressbar  # type: ignore
from ttkbootstrap.scrolled import ScrolledText  # type: ignore

from sticker_convert.gui_components.frames.right_clicker import RightClicker

if TYPE_CHECKING:
    from sticker_convert.gui import GUI  # type: ignore


class ProgressFrame(LabelFrame):
    progress_bar_cli = None
    progress_bar_steps = 0
    auto_scroll = True

    def __init__(self, gui: "GUI", *args: Any, **kwargs: Any) -> None:
        self.gui = gui
        super().__init__(*args, **kwargs)  # type: ignore

        self.message_box = ScrolledText(self, height=15, wrap="word")
        self.message_box._text.bind("<Button-3><ButtonRelease-3>", RightClicker)  # type: ignore
        self.message_box._text.config(state="disabled")  # type: ignore
        self.progress_bar = Progressbar(self, orient="horizontal", mode="determinate")

        self.message_box.bind("<Enter>", self.cb_disable_autoscroll)
        self.message_box.bind("<Leave>", self.cb_enable_autoscroll)

        self.message_box.pack(expand=True, fill="x")
        self.progress_bar.pack(expand=True, fill="x")

    def update_progress_bar(
        self,
        set_progress_mode: Optional[str] = None,
        steps: int = 0,
        update_bar: bool = False,
    ) -> None:
        if update_bar and self.progress_bar_cli:
            self.progress_bar_cli.update()
            self.progress_bar["value"] += 100 / self.progress_bar_steps
        elif set_progress_mode == "determinate":
            self.progress_bar_cli = tqdm(total=steps)
            self.progress_bar.config(mode="determinate")
            self.progress_bar_steps = steps
            self.progress_bar.stop()
            self.progress_bar["value"] = 0
        elif set_progress_mode == "indeterminate":
            if self.progress_bar_cli:
                self.progress_bar_cli.close()
                self.progress_bar_cli = None
            self.progress_bar["value"] = 0
            self.progress_bar.config(mode="indeterminate")
            self.progress_bar.start(50)
        elif set_progress_mode == "clear":
            if self.progress_bar_cli:
                self.progress_bar_cli.reset()
            self.progress_bar.config(mode="determinate")
            self.progress_bar.stop()
            self.progress_bar["value"] = 0

    def update_message_box(self, *args: Any, **kwargs: Any) -> None:
        msg = kwargs.get("msg")
        cls = kwargs.get("cls")
        file = kwargs.get("file")

        if not msg and len(args) == 1:
            msg = str(args[0])

        if msg:
            if self.progress_bar_cli:
                self.progress_bar_cli.write(msg)
            elif file:
                print(msg, file=file)
            else:
                print(msg)
            msg += "\n"

        self.message_box._text.config(state="normal")  # type: ignore

        if cls:
            self.message_box.delete(1.0, "end")  # type: ignore

        if msg:
            self.message_box.insert("end", msg)  # type: ignore

            if self.auto_scroll:
                self.message_box._text.yview_moveto(1.0)  # type: ignore

        self.message_box._text.config(state="disabled")  # type: ignore

    def cb_disable_autoscroll(self, *_: Any) -> None:
        self.auto_scroll = False

    def cb_enable_autoscroll(self, *_: Any) -> None:
        self.auto_scroll = True



#!/usr/bin/env python3
import json
import secrets
import uuid
from typing import Any, Callable, Dict, Optional
from urllib.parse import parse_qs, urlparse

import requests

from sticker_convert.job_option import CredOption


class GetKakaoAuth:
    def __init__(
        self,
        opt_cred: CredOption,
        cb_msg: Callable[..., None] = print,
        cb_msg_block: Callable[..., Any] = input,
        cb_ask_str: Callable[..., str] = input,
    ) -> None:
        self.username = opt_cred.kakao_username
        self.password = opt_cred.kakao_password
        self.country_code = opt_cred.kakao_country_code
        self.phone_number = opt_cred.kakao_phone_number

        self.cb_msg = cb_msg
        self.cb_msg_block = cb_msg_block
        self.cb_ask_str = cb_ask_str

        self.device_uuid = secrets.token_hex(32)
        self.device_ssaid = secrets.token_hex(20)
        self.uuid_c = str(uuid.uuid4())
        self.device_info = (
            f"android/30; uuid={self.device_uuid}; ssaid={self.device_ssaid}; "
            + 'model=ANDROID-SDK-BUILT-FOR-X86; screen_resolution=1080x1920; sim=310260/1/us; onestore=false; uvc2={"volume":5,"network_operator":"310260","is_roaming":"false","va":[],"brightness":102,"totalMemory":30866040,"batteryPct":1,"webviewVersion":"83.0.4103.106"}'
        )
        self.app_platform = "android"
        self.app_version_number = "10.0.3"
        self.app_language = "en"
        self.app_version = (
            f"{self.app_platform}/{self.app_version_number}/{self.app_language}"
        )

        self.headers = {
            "Host": "katalk.kakao.com",
            "Accept-Language": "en",
            "User-Agent": "KT/10.0.3 An/11 en",
            "Device-Info": self.device_info,
            "A": self.app_version,
            "C": self.uuid_c,
            "Content-Type": "application/json",
            "Connection": "close",
        }

    def login(self) -> bool:
        self.cb_msg("Logging in")

        json_data = {
            "id": self.username,
            "password": self.password,
        }

        response = requests.post(
            "https://katalk.kakao.com/android/account2/login",
            headers=self.headers,
            json=json_data,
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at login: {response.text}")
            return False

        self.headers["Ss"] = response.headers["Set-SS"]
        self.country_dicts = response_json["viewData"]["countries"]["all"]

        return True

    def get_country_iso(self) -> bool:
        self.country_iso = None
        for country_dict in self.country_dicts:
            if country_dict["code"] == self.country_code:
                self.country_iso = country_dict["iso"]

        if not self.country_iso:
            self.cb_msg_block("Invalid country code")
            return False

        return True

    def enter_phone(self) -> bool:
        self.cb_msg("Submitting phone number")

        json_data: Dict[str, Any] = {
            "countryCode": self.country_code,
            "countryIso": self.country_iso,
            "phoneNumber": self.phone_number,
            "method": "sms",
            "termCodes": [],
            "simPhoneNumber": f"+{self.country_code}{self.phone_number}",
        }

        response = requests.post(
            "https://katalk.kakao.com/android/account2/phone-number",
            headers=self.headers,
            json=json_data,
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at entering phone number: {response.text}")
            return False

        self.verify_method = response_json["view"]

        if self.verify_method == "passcode":
            return self.verify_receive_sms()
        if self.verify_method == "mo-send":
            dest_number = response_json["viewData"]["moNumber"]
            msg = response_json["viewData"]["moMessage"]
            return self.verify_send_sms(dest_number, msg)
        self.cb_msg_block(f"Unknown verification method: {response.text}")
        return False

    def verify_send_sms(self, dest_number: str, msg: str) -> bool:
        self.cb_msg("Verification by sending SMS")

        response = requests.post(
            "https://katalk.kakao.com/android/account2/mo-sent", headers=self.headers
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at confirm sending SMS: {response.text}")
            return False

        prompt = f"Send this SMS message to number {dest_number} then press enter:"
        self.cb_msg(msg)
        if self.cb_ask_str != input:
            self.cb_ask_str(prompt, initialvalue=msg, cli_show_initialvalue=False)
        else:
            input(prompt)

        response = requests.post(
            "https://katalk.kakao.com/android/account2/mo-confirm", headers=self.headers
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at verifying SMS sent: {response.text}")
            return False

        if response_json.get("reason") or "error" in response_json.get("message", ""):
            self.cb_msg_block(f"Failed at verifying SMS sent: {response.text}")
            return False

        self.confirm_url = response_json.get("viewData", {}).get("url")

        return True

    def verify_receive_sms(self) -> bool:
        self.cb_msg("Verification by receiving SMS")

        passcode = self.cb_ask_str("Enter passcode received from SMS:")

        json_data = {
            "passcode": passcode,
        }

        response = requests.post(
            "https://katalk.kakao.com/android/account2/passcode",
            headers=self.headers,
            json=json_data,
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at verifying passcode: {response.text}")
            return False

        self.confirm_url = response_json.get("viewData", {}).get("url")

        return True

    def confirm_device_change(self) -> bool:
        self.cb_msg("Confirm device change")

        confirm_url_parsed = urlparse(self.confirm_url)  # type: ignore
        confirm_url_qs = parse_qs(confirm_url_parsed.query)  # type: ignore
        session_token: str = confirm_url_qs["sessionToken"][0]  # type: ignore

        headers_browser = {
            "Host": "katalk.kakao.com",
            "Accept": "*/*",
            "X-Requested-With": "XMLHttpRequest",
            "User-Agent": "Mozilla/5.0 (Linux; Android 11; Android SDK built for x86 Build/RSR1.210210.001.A1; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/83.0.4103.106 Mobile Safari/537.36;KAKAOTALK 2410030",
            "Content-Type": "application/json",
            "Origin": "https://katalk.kakao.com",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": self.confirm_url,
            "Accept-Language": "en-US,en;q=0.9",
            "Connection": "close",
        }

        json_data: Dict[str, str] = {
            "decision": "continue",
            "lang": self.app_language,
            "sessionToken": session_token,
            "appVersion": self.app_version_number,
        }

        response = requests.post(
            "https://katalk.kakao.com/android/account2/confirm-device-change",
            headers=headers_browser,
            json=json_data,
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at confirm device change: {response.text}")
            return False

        return True

    def passcode_callback(self) -> bool:
        self.cb_msg("Passcode callback")

        response = requests.get(
            "https://katalk.kakao.com/android/account2/passcode/callback",
            headers=self.headers,
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at passcode callback: {response.text}")
            return False

        self.nickname = response_json.get("viewData", {}).get("nickname")

        if self.nickname is None:
            self.cb_msg_block(f"Failed at passcode callback: {response.text}")
            return False

        return True

    def get_profile(self) -> bool:
        self.cb_msg("Get profile")

        json_data = {
            "nickname": self.nickname,
            "profileImageFlag": 1,
            "friendAutomation": True,
        }

        response = requests.post(
            "https://katalk.kakao.com/android/account2/profile",
            headers=self.headers,
            json=json_data,
        )

        response_json = json.loads(response.text)

        if response_json["status"] != 0:
            self.cb_msg_block(f"Failed at get profile: {response.text}")
            return False

        self.access_token = response_json["signupData"]["oauth2Token"]["accessToken"]

        return True

    def get_cred(self) -> Optional[str]:
        self.cb_msg("Get authorization token")

        authorization_token = None

        steps = (
            self.login,
            self.get_country_iso,
            self.enter_phone,
            self.confirm_device_change,
            self.passcode_callback,
            self.get_profile,
        )

        for step in steps:
            success = step()
            if not success:
                return None

        authorization_token = self.access_token + "-" + self.device_uuid
        return authorization_token



#!/usr/bin/env python3
from __future__ import annotations

from pathlib import Path
from queue import Queue
from typing import Any, List, Optional, Tuple, Union

import requests

from sticker_convert.job_option import CredOption
from sticker_convert.utils.callback import Callback, CallbackReturn, CbQueueItemType


class DownloadBase:
    def __init__(
        self,
        url: str,
        out_dir: Path,
        opt_cred: Optional[CredOption],
        cb: Union[
            Queue[CbQueueItemType],
            Callback,
        ],
        cb_return: CallbackReturn,
    ) -> None:
        self.url = url
        self.out_dir = out_dir
        self.opt_cred = opt_cred
        self.cb = cb
        self.cb_return = cb_return

    def download_multiple_files(
        self, targets: List[Tuple[str, Path]], retries: int = 3, **kwargs: Any
    ) -> None:
        # targets format: [(url1, dest2), (url2, dest2), ...]
        self.cb.put(
            ("bar", None, {"set_progress_mode": "determinate", "steps": len(targets)})
        )

        for url, dest in targets:
            self.download_file(url, dest, retries, show_progress=False, **kwargs)

            self.cb.put("update_bar")

    def download_file(
        self,
        url: str,
        dest: Optional[Path] = None,
        retries: int = 3,
        show_progress: bool = True,
        **kwargs: Any,
    ) -> bytes:
        result = b""
        chunk_size = 102400

        for retry in range(retries):
            try:
                response = requests.get(url, stream=True, **kwargs)
                total_length = int(response.headers.get("content-length"))  # type: ignore

                if response.status_code != 200:
                    return b""
                self.cb.put(f"Downloading {url}")

                if show_progress:
                    steps = (total_length / chunk_size) + 1
                    self.cb.put(
                        (
                            "bar",
                            None,
                            {"set_progress_mode": "determinate", "steps": int(steps)},
                        )
                    )

                for chunk in response.iter_content(chunk_size=chunk_size):
                    if chunk:
                        result += chunk
                        if show_progress:
                            self.cb.put("update_bar")

                break
            except requests.exceptions.RequestException:
                msg = f"Cannot download {url} (tried {retry+1}/{retries} times)"
                self.cb.put(msg)

        if not result:
            return b""
        if dest:
            with open(dest, "wb+") as f:
                f.write(result)
            msg = f"Downloaded {url}"
            self.cb.put(msg)
            return b""
        return result



#!/usr/bin/env python3
import platform
import shutil
import subprocess
from pathlib import Path
from typing import Any, Callable, List, Tuple, Union


class RunBin:
    @staticmethod
    def get_bin(
        executable: str, silent: bool = False, cb_msg: Callable[..., Any] = print
    ) -> Union[str, None]:
        if Path(executable).is_file():
            return executable

        if platform.system() == "Windows":
            executable = executable + ".exe"

        which_result = shutil.which(executable)
        if which_result is not None:
            return str(Path(which_result).resolve())
        if silent is False:
            cb_msg(f"Warning: Cannot find binary file {executable}")

        return None

    @staticmethod
    def run_cmd(
        cmd_list: List[str], silence: bool = False, cb_msg: Callable[..., Any] = print
    ) -> Tuple[bool, str]:
        bin_path = RunBin.get_bin(cmd_list[0])

        if bin_path:
            cmd_list[0] = bin_path
        else:
            if silence is False:
                cb_msg(
                    f"Error while executing {' '.join(cmd_list)} : Command not found"
                )
            return False, ""

        # sp = subprocess.Popen(cmd_list, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
        sp = subprocess.run(
            cmd_list,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=False,
        )

        output_str = sp.stdout.decode()
        error_str = sp.stderr.decode()

        if silence is False and error_str != "":
            cb_msg(f"Error while executing {' '.join(cmd_list)} : {error_str}")
            return False, ""

        return True, output_str



#!/usr/bin/env python3
from typing import TYPE_CHECKING, Any

from ttkbootstrap import Button, Checkbutton, Entry, Label, LabelFrame, OptionMenu  # type: ignore

from sticker_convert.gui_components.frames.right_clicker import RightClicker
from sticker_convert.gui_components.windows.advanced_compression_window import AdvancedCompressionWindow

if TYPE_CHECKING:
    from sticker_convert.gui import GUI  # type: ignore


class CompFrame(LabelFrame):
    def __init__(self, gui: "GUI", *args: Any, **kwargs: Any) -> None:
        self.gui = gui
        super().__init__(*args, **kwargs)

        self.grid_columnconfigure(2, weight=1)

        self.no_compress_help_btn = Button(
            self,
            text="?",
            width=1,
            command=lambda: self.gui.cb_msg_block(self.gui.help["comp"]["no_compress"]),
            bootstyle="secondary",  # type: ignore
        )
        self.no_compress_lbl = Label(self, text="No compression")
        self.no_compress_cbox = Checkbutton(
            self,
            variable=self.gui.no_compress_var,
            command=self.cb_no_compress,
            onvalue=True,
            offvalue=False,
            bootstyle="danger-round-toggle",  # type: ignore
        )

        self.comp_preset_help_btn = Button(
            self,
            text="?",
            width=1,
            command=lambda: self.gui.cb_msg_block(self.gui.help["comp"]["preset"]),
            bootstyle="secondary",  # type: ignore
        )
        self.comp_preset_lbl = Label(self, text="Preset")
        self.comp_preset_opt = OptionMenu(
            self,
            self.gui.comp_preset_var,
            self.gui.comp_preset_var.get(),
            *self.gui.compression_presets.keys(),
            command=self.cb_comp_apply_preset,
            bootstyle="secondary",  # type: ignore
        )
        self.comp_preset_opt.config(width=15)

        self.steps_help_btn = Button(
            self,
            text="?",
            width=1,
            command=lambda: self.gui.cb_msg_block(self.gui.help["comp"]["steps"]),
            bootstyle="secondary",  # type: ignore
        )
        self.steps_lbl = Label(self, text="Number of steps")
        self.steps_entry = Entry(self, textvariable=self.gui.steps_var, width=8)
        self.steps_entry.bind("<Button-3><ButtonRelease-3>", RightClicker)

        self.processes_help_btn = Button(
            self,
            text="?",
            width=1,
            command=lambda: self.gui.cb_msg_block(self.gui.help["comp"]["processes"]),
            bootstyle="secondary",  # type: ignore
        )
        self.processes_lbl = Label(self, text="Number of processes")
        self.processes_entry = Entry(self, textvariable=self.gui.processes_var, width=8)
        self.processes_entry.bind("<Button-3><ButtonRelease-3>", RightClicker)

        self.comp_advanced_btn = Button(
            self,
            text="Advanced...",
            command=self.cb_compress_advanced,
            bootstyle="secondary",  # type: ignore
        )

        self.no_compress_help_btn.grid(column=0, row=0, sticky="w", padx=3, pady=3)
        self.no_compress_lbl.grid(column=1, row=0, sticky="w", padx=3, pady=3)
        self.no_compress_cbox.grid(column=2, row=0, sticky="nes", padx=3, pady=3)

        self.comp_preset_help_btn.grid(column=0, row=1, sticky="w", padx=3, pady=3)
        self.comp_preset_lbl.grid(column=1, row=1, sticky="w", padx=3, pady=3)
        self.comp_preset_opt.grid(column=2, row=1, sticky="nes", padx=3, pady=3)

        self.steps_help_btn.grid(column=0, row=2, sticky="w", padx=3, pady=3)
        self.steps_lbl.grid(column=1, row=2, sticky="w", padx=3, pady=3)
        self.steps_entry.grid(column=2, row=2, sticky="nes", padx=3, pady=3)

        self.processes_help_btn.grid(column=0, row=3, sticky="w", padx=3, pady=3)
        self.processes_lbl.grid(column=1, row=3, sticky="w", padx=3, pady=3)
        self.processes_entry.grid(column=2, row=3, sticky="nes", padx=3, pady=3)

        self.comp_advanced_btn.grid(column=2, row=4, sticky="nes", padx=3, pady=3)

        self.cb_comp_apply_preset()
        self.cb_no_compress()

    def cb_comp_apply_preset(self, *_: Any) -> None:
        selection = self.gui.get_preset()
        if selection == "auto":
            if self.gui.get_input_name() == "local":
                self.gui.no_compress_var.set(True)
            else:
                self.gui.no_compress_var.set(False)

        self.gui.fps_min_var.set(self.gui.compression_presets[selection]["fps"]["min"])
        self.gui.fps_max_var.set(self.gui.compression_presets[selection]["fps"]["max"])
        self.gui.fps_power_var.set(
            self.gui.compression_presets[selection]["fps"]["power"]
        )
        self.gui.res_w_min_var.set(
            self.gui.compression_presets[selection]["res"]["w"]["min"]
        )
        self.gui.res_w_max_var.set(
            self.gui.compression_presets[selection]["res"]["w"]["max"]
        )
        self.gui.res_h_min_var.set(
            self.gui.compression_presets[selection]["res"]["h"]["min"]
        )
        self.gui.res_h_max_var.set(
            self.gui.compression_presets[selection]["res"]["h"]["max"]
        )
        self.gui.res_power_var.set(
            self.gui.compression_presets[selection]["res"]["power"]
        )
        self.gui.quality_min_var.set(
            self.gui.compression_presets[selection]["quality"]["min"]
        )
        self.gui.quality_max_var.set(
            self.gui.compression_presets[selection]["quality"]["max"]
        )
        self.gui.quality_power_var.set(
            self.gui.compression_presets[selection]["quality"]["power"]
        )
        self.gui.color_min_var.set(
            self.gui.compression_presets[selection]["color"]["min"]
        )
        self.gui.color_max_var.set(
            self.gui.compression_presets[selection]["color"]["max"]
        )
        self.gui.color_power_var.set(
            self.gui.compression_presets[selection]["color"]["power"]
        )
        self.gui.duration_min_var.set(
            self.gui.compression_presets[selection]["duration"]["min"]
        )
        self.gui.duration_max_var.set(
            self.gui.compression_presets[selection]["duration"]["max"]
        )
        self.gui.img_size_max_var.set(
            self.gui.compression_presets[selection]["size_max"]["img"]
        )
        self.gui.vid_size_max_var.set(
            self.gui.compression_presets[selection]["size_max"]["vid"]
        )
        self.gui.img_format_var.set(
            self.gui.compression_presets[selection]["format"]["img"]
        )
        self.gui.vid_format_var.set(
            self.gui.compression_presets[selection]["format"]["vid"]
        )
        self.gui.fake_vid_var.set(self.gui.compression_presets[selection]["fake_vid"])
        self.gui.scale_filter_var.set(
            self.gui.compression_presets[selection]["scale_filter"]
        )
        self.gui.quantize_method_var.set(
            self.gui.compression_presets[selection]["quantize_method"]
        )
        self.gui.default_emoji_var.set(
            self.gui.compression_presets[selection]["default_emoji"]
        )
        self.gui.steps_var.set(self.gui.compression_presets[selection]["steps"])

        self.cb_no_compress()
        self.gui.highlight_fields()

    def cb_compress_advanced(self, *_: Any) -> None:
        AdvancedCompressionWindow(self.gui)

    def cb_no_compress(self, *_: Any) -> None:
        if self.gui.no_compress_var.get() is True:
            state = "disabled"
        else:
            state = "normal"

        self.comp_advanced_btn.config(state=state)
        self.steps_entry.config(state=state)
        self.processes_entry.config(state=state)

    def set_inputs_comp(self, state: str) -> None:
        self.comp_preset_opt.config(state=state)
        self.comp_advanced_btn.config(state=state)
        self.steps_entry.config(state=state)
        self.processes_entry.config(state=state)

    def set_states(self, state: str) -> None:
        self.no_compress_cbox.config(state=state)
        self.set_inputs_comp(state=state)



#!/usr/bin/env python3
from __future__ import annotations

import json
import zipfile
from io import BytesIO
from pathlib import Path
from queue import Queue
from typing import Any, List, Optional, Tuple, Union
from urllib.parse import urlparse

import requests
from bs4 import BeautifulSoup
from bs4.element import Tag

from sticker_convert.downloaders.download_base import DownloadBase
from sticker_convert.job_option import CredOption
from sticker_convert.utils.callback import Callback, CallbackReturn, CbQueueItemType
from sticker_convert.utils.files.metadata_handler import MetadataHandler
from sticker_convert.utils.media.decrypt_kakao import DecryptKakao


class MetadataKakao:
    @staticmethod
    def get_info_from_share_link(url: str) -> Tuple[Optional[str], Optional[str]]:
        headers = {"User-Agent": "Android"}

        response = requests.get(url, headers=headers)
        soup = BeautifulSoup(response.text, "html.parser")

        pack_title_tag = soup.find("title")  # type: ignore
        if not pack_title_tag:
            return None, None

        pack_title: str = pack_title_tag.string  # type: ignore

        app_scheme_link_tag = soup.find("a", id="app_scheme_link")  # type: ignore
        assert isinstance(app_scheme_link_tag, Tag)

        data_urls = app_scheme_link_tag.get("data-url")
        if not data_urls:
            return None, None
        if isinstance(data_urls, list):
            data_url = data_urls[0]
        else:
            data_url = data_urls

        item_code = data_url.replace("kakaotalk://store/emoticon/", "").split("?")[0]

        return pack_title, item_code

    @staticmethod
    def get_info_from_pack_title(
        pack_title: str,
    ) -> Tuple[Optional[str], Optional[str], Optional[str]]:
        pack_meta_r = requests.get(f"https://e.kakao.com/api/v1/items/t/{pack_title}")

        if pack_meta_r.status_code == 200:
            pack_meta = json.loads(pack_meta_r.text)
        else:
            return None, None, None

        author = pack_meta["result"]["artist"]
        title_ko = pack_meta["result"]["title"]
        thumbnail_urls = pack_meta["result"]["thumbnailUrls"]

        return author, title_ko, thumbnail_urls

    @staticmethod
    def get_item_code(title_ko: str, auth_token: str) -> Optional[str]:
        headers = {
            "Authorization": auth_token,
        }

        data = {"query": title_ko}

        response = requests.post(
            "https://talk-pilsner.kakao.com/emoticon/item_store/instant_search",
            headers=headers,
            data=data,
        )

        if response.status_code != 200:
            return None

        response_json = json.loads(response.text)
        item_code = response_json["emoticons"][0]["item_code"]

        return item_code

    @staticmethod
    def get_title_from_id(item_code: str, auth_token: str) -> Optional[str]:
        headers = {
            "Authorization": auth_token,
        }

        response = requests.post(
            f"https://talk-pilsner.kakao.com/emoticon/api/store/v3/items/{item_code}",
            headers=headers,
        )

        if response.status_code != 200:
            return None

        response_json = json.loads(response.text)
        title = response_json["itemUnitInfo"][0]["title"]
        # play_path_format = response_json['itemUnitInfo'][0]['playPathFormat']
        # stickers_count = len(response_json['itemUnitInfo'][0]['sizes'])

        return title


class DownloadKakao(DownloadBase):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.pack_title: Optional[str] = None
        self.author: Optional[str] = None

    def download_stickers_kakao(self) -> bool:
        auth_token = None
        if self.opt_cred:
            auth_token = self.opt_cred.kakao_auth_token

        if urlparse(self.url).netloc == "emoticon.kakao.com":
            self.pack_title, item_code = MetadataKakao.get_info_from_share_link(
                self.url
            )

            if item_code:
                return self.download_animated(item_code)
            self.cb.put("Download failed: Cannot download metadata for sticker pack")
            return False

        if self.url.isnumeric() or self.url.startswith("kakaotalk://store/emoticon/"):
            item_code = self.url.replace("kakaotalk://store/emoticon/", "")

            self.pack_title = None
            if auth_token:
                self.pack_title = MetadataKakao.get_title_from_id(item_code, auth_token)
                if not self.pack_title:
                    self.cb.put("Warning: Cannot get pack_title with auth_token.")
                    self.cb.put(
                        "Is auth_token invalid / expired? Try to regenerate it."
                    )
                    self.cb.put("Continuing without getting pack_title")

            return self.download_animated(item_code)

        if urlparse(self.url).netloc == "e.kakao.com":
            self.pack_title = self.url.replace("https://e.kakao.com/t/", "")
            (
                self.author,
                title_ko,
                thumbnail_urls,
            ) = MetadataKakao.get_info_from_pack_title(self.pack_title)

            assert self.author
            assert title_ko
            assert thumbnail_urls

            if not thumbnail_urls:
                self.cb.put(
                    "Download failed: Cannot download metadata for sticker pack"
                )
                return False

            if auth_token:
                item_code = MetadataKakao.get_item_code(title_ko, auth_token)
                if item_code:
                    return self.download_animated(item_code)
                msg = "Warning: Cannot get item code.\n"
                msg += "Is auth_token invalid / expired? Try to regenerate it.\n"
                msg += "Continue to download static stickers instead?"
                self.cb.put(("ask_bool", (msg,), None))
                if self.cb_return:
                    response = self.cb_return.get_response()
                else:
                    response = False

                if response is False:
                    return False

            return self.download_static(thumbnail_urls)

        self.cb.put("Download failed: Unrecognized URL")
        return False

    def download_static(self, thumbnail_urls: str) -> bool:
        MetadataHandler.set_metadata(
            self.out_dir, title=self.pack_title, author=self.author
        )

        targets: List[Tuple[str, Path]] = []

        for num, url in enumerate(thumbnail_urls):
            dest = Path(self.out_dir, str(num).zfill(3) + ".png")
            targets.append((url, dest))

        self.download_multiple_files(targets)

        return True

    def download_animated(self, item_code: str) -> bool:
        MetadataHandler.set_metadata(
            self.out_dir, title=self.pack_title, author=self.author
        )

        pack_url = f"http://item.kakaocdn.net/dw/{item_code}.file_pack.zip"

        zip_file = self.download_file(pack_url)
        if zip_file:
            self.cb.put(f"Downloaded {pack_url}")
        else:
            self.cb.put(f"Cannot download {pack_url}")
            return False

        with zipfile.ZipFile(BytesIO(zip_file)) as zf:
            self.cb.put("Unzipping...")
            self.cb.put(
                (
                    "bar",
                    None,
                    {"set_progress_mode": "determinate", "steps": len(zf.namelist())},
                )
            )

            for num, f_path in enumerate(sorted(zf.namelist())):
                ext = Path(f_path).suffix

                if ext in (".gif", ".webp"):
                    data = DecryptKakao.xor_data(zf.read(f_path))
                    self.cb.put(f"Decrypted {f_path}")
                else:
                    data = zf.read(f_path)
                    self.cb.put(f"Read {f_path}")

                out_path = Path(self.out_dir, str(num).zfill(3) + ext)
                with open(out_path, "wb") as f:
                    f.write(data)

                self.cb.put("update_bar")

        self.cb.put(f"Finished getting {pack_url}")

        return True

    @staticmethod
    def start(
        url: str,
        out_dir: Path,
        opt_cred: Optional[CredOption],
        cb: "Union[Queue[CbQueueItemType], Callback]",
        cb_return: CallbackReturn,
    ) -> bool:
        downloader = DownloadKakao(url, out_dir, opt_cred, cb, cb_return)
        return downloader.download_stickers_kakao()



#!/usr/bin/env python3
import copy
import shutil
import zipfile
from pathlib import Path
from queue import Queue
from typing import Any, List, Union

from sticker_convert.converter import StickerConvert
from sticker_convert.job_option import CompOption, CredOption, OutputOption
from sticker_convert.uploaders.upload_base import UploadBase
from sticker_convert.utils.callback import Callback, CallbackReturn, CbQueueItemType
from sticker_convert.utils.files.cache_store import CacheStore
from sticker_convert.utils.files.metadata_handler import MetadataHandler
from sticker_convert.utils.files.sanitize_filename import sanitize_filename
from sticker_convert.utils.media.codec_info import CodecInfo
from sticker_convert.utils.media.format_verify import FormatVerify


class CompressWastickers(UploadBase):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.base_spec.size_max_img = 100000
        self.base_spec.size_max_vid = 500000
        self.base_spec.duration_min = 8
        self.base_spec.duration_max = 10000
        self.base_spec.square = True
        self.base_spec.set_res(512)
        self.base_spec.set_format((".webp",))

        self.spec_cover = CompOption(
            size_max_img=50000,
            size_max_vid=50000,
            animated=False,
        )
        self.spec_cover.set_res(96)
        self.spec_cover.set_res(96)
        self.spec_cover.set_fps(0)
        self.spec_cover.set_format((".png",))

        self.webp_spec = copy.deepcopy(self.base_spec)
        self.webp_spec.set_format((".webp",))
        self.webp_spec.animated = None if self.opt_comp.fake_vid else True

        self.png_spec = copy.deepcopy(self.base_spec)
        self.png_spec.set_format((".png",))
        self.png_spec.animated = False

        self.opt_comp_merged = copy.deepcopy(self.opt_comp)
        self.opt_comp_merged.merge(self.base_spec)

    def compress_wastickers(self) -> List[str]:
        urls: List[str] = []
        title, author, _ = MetadataHandler.get_metadata(
            self.opt_output.dir,
            title=self.opt_output.title,
            author=self.opt_output.author,
        )
        if not title:
            self.cb.put("Title is required for compressing .wastickers")
            return urls
        if not author:
            self.cb.put("Author is required for compressing .wastickers")
            return urls
        packs = MetadataHandler.split_sticker_packs(
            self.opt_output.dir,
            title=title,
            file_per_pack=30,
            separate_image_anim=not self.opt_comp.fake_vid,
        )

        for pack_title, stickers in packs.items():
            # Originally the Sticker Maker application name the files with int(time.time())
            with CacheStore.get_cache_store(path=self.opt_comp.cache_dir) as tempdir:
                for num, src in enumerate(stickers):
                    self.cb.put(f"Verifying {src} for compressing into .wastickers")

                    if self.opt_comp.fake_vid or CodecInfo.is_anim(src):
                        ext = ".webp"
                    else:
                        ext = ".png"

                    dst = Path(tempdir, str(num) + ext)

                    if FormatVerify.check_file(
                        src, spec=self.webp_spec
                    ) or FormatVerify.check_file(src, spec=self.png_spec):
                        shutil.copy(src, dst)
                    else:
                        StickerConvert.convert(
                            Path(src),
                            Path(dst),
                            self.opt_comp_merged,
                            self.cb,
                            self.cb_return,
                        )

                out_f = Path(
                    self.opt_output.dir, sanitize_filename(pack_title + ".wastickers")
                ).as_posix()

                self.add_metadata(Path(tempdir), pack_title, author)
                with zipfile.ZipFile(out_f, "w", zipfile.ZIP_DEFLATED) as zipf:
                    for file in Path(tempdir).iterdir():
                        file_path = Path(tempdir, file.name)
                        zipf.write(file_path, arcname=file_path.name)

            self.cb.put((out_f))
            urls.append(out_f)

        return urls

    def add_metadata(self, pack_dir: Path, title: str, author: str) -> None:
        opt_comp_merged = copy.deepcopy(self.opt_comp)
        opt_comp_merged.merge(self.spec_cover)

        cover_path_old = MetadataHandler.get_cover(self.opt_output.dir)
        cover_path_new = Path(pack_dir, "100.png")
        if cover_path_old:
            if FormatVerify.check_file(cover_path_old, spec=self.spec_cover):
                shutil.copy(cover_path_old, cover_path_new)
            else:
                StickerConvert.convert(
                    cover_path_old,
                    cover_path_new,
                    opt_comp_merged,
                    self.cb,
                    self.cb_return,
                )
        else:
            # First image in the directory, extracting first frame
            first_image = [
                i
                for i in sorted(self.opt_output.dir.iterdir())
                if Path(self.opt_output.dir, i.name).is_file()
                and i.suffix not in (".txt", ".m4a", ".wastickers")
            ][0]
            StickerConvert.convert(
                Path(self.opt_output.dir, first_image),
                cover_path_new,
                opt_comp_merged,
                self.cb,
                self.cb_return,
            )

        MetadataHandler.set_metadata(pack_dir, author=author, title=title)

    @staticmethod
    def start(
        opt_output: OutputOption,
        opt_comp: CompOption,
        opt_cred: CredOption,
        cb: "Union[Queue[CbQueueItemType], Callback]",
        cb_return: CallbackReturn,
    ) -> List[str]:
        exporter = CompressWastickers(opt_output, opt_comp, opt_cred, cb, cb_return)
        return exporter.compress_wastickers()



#!/usr/bin/env python3
import copy
from pathlib import Path
from queue import Queue
from typing import Any, Dict, List, Union

import anyio
from signalstickers_client import StickersClient  # type: ignore
from signalstickers_client.errors import SignalException  # type: ignore
from signalstickers_client.models import LocalStickerPack, Sticker  # type: ignore

from sticker_convert.converter import StickerConvert
from sticker_convert.job_option import CompOption, CredOption, OutputOption
from sticker_convert.uploaders.upload_base import UploadBase
from sticker_convert.utils.callback import Callback, CallbackReturn, CbQueueItemType
from sticker_convert.utils.files.metadata_handler import MetadataHandler
from sticker_convert.utils.media.codec_info import CodecInfo
from sticker_convert.utils.media.format_verify import FormatVerify


class UploadSignal(UploadBase):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

        self.base_spec.set_size_max(300000)
        self.base_spec.set_res_max(512)
        self.base_spec.duration_max = 3000
        self.base_spec.square = True

        self.png_spec = copy.deepcopy(self.base_spec)
        self.png_spec.set_format((".apng",))

        self.webp_spec = copy.deepcopy(self.base_spec)
        self.webp_spec.format_img = (".webp",)
        self.webp_spec.animated = False

        self.opt_comp_merged = copy.deepcopy(self.opt_comp)
        self.opt_comp_merged.merge(self.base_spec)

    @staticmethod
    async def upload_pack(pack: LocalStickerPack, uuid: str, password: str) -> str:
        async with StickersClient(uuid, password) as client:
            pack_id, pack_key = await client.upload_pack(pack)  # type: ignore

        result = (
            f"https://signal.art/addstickers/#pack_id={pack_id}&pack_key={pack_key}"
        )
        return result

    def add_stickers_to_pack(
        self, pack: LocalStickerPack, stickers: List[Path], emoji_dict: Dict[str, str]
    ) -> None:
        for src in stickers:
            self.cb.put(f"Verifying {src} for uploading to signal")

            sticker = Sticker()
            sticker.id = pack.nb_stickers  # type: ignore

            emoji = emoji_dict.get(Path(src).stem, None)
            if not emoji:
                self.cb.put(
                    f"Warning: Cannot find emoji for file {Path(src).name}, skip uploading this file..."
                )
                continue
            sticker.emoji = emoji[:1]  # type: ignore

            if Path(src).suffix == ".webp":
                spec_choice = self.webp_spec
            else:
                spec_choice = self.png_spec

            if not FormatVerify.check_file(src, spec=spec_choice):
                if self.opt_comp.fake_vid or CodecInfo.is_anim(src):
                    dst = "bytes.apng"
                else:
                    dst = "bytes.png"
                success, _, image_data, _ = StickerConvert.convert(
                    Path(src), Path(dst), self.opt_comp_merged, self.cb, self.cb_return
                )
                if not success:
                    self.cb.put(
                        f"Warning: Cannot compress file {Path(src).name}, skip uploading this file..."
                    )
                    continue

                assert isinstance(image_data, bytes)

                sticker.image_data = image_data  # type: ignore
            else:
                with open(src, "rb") as f:
                    sticker.image_data = f.read()  # type: ignore

            pack._addsticker(sticker)  # type: ignore

    def upload_stickers_signal(self) -> List[str]:
        urls: List[str] = []

        if not self.opt_cred.signal_uuid:
            self.cb.put("uuid required for uploading to Signal")
            return urls
        if not self.opt_cred.signal_password:
            self.cb.put("password required for uploading to Signal")
            return urls

        title, author, emoji_dict = MetadataHandler.get_metadata(
            self.opt_output.dir,
            title=self.opt_output.title,
            author=self.opt_output.author,
        )
        if title is None:
            raise TypeError(f"title cannot be {title}")
        if author is None:
            raise TypeError(f"author cannot be {author}")
        if emoji_dict is None:
            msg_block = "emoji.txt is required for uploading signal stickers\n"
            msg_block += f"emoji.txt generated for you in {self.opt_output.dir}\n"
            msg_block += f"Default emoji is set to {self.opt_comp.default_emoji}.\n"
            msg_block += "Please edit emoji.txt now, then continue"
            MetadataHandler.generate_emoji_file(
                directory=self.opt_output.dir, default_emoji=self.opt_comp.default_emoji
            )

            self.cb.put(("msg_block", (msg_block,), None))
            if self.cb_return:
                self.cb_return.get_response()

            title, author, emoji_dict = MetadataHandler.get_metadata(
                self.opt_output.dir,
                title=self.opt_output.title,
                author=self.opt_output.author,
            )
            assert title
            assert author
            assert emoji_dict

        packs = MetadataHandler.split_sticker_packs(
            self.opt_output.dir,
            title=title,
            file_per_pack=200,
            separate_image_anim=False,
        )
        for pack_title, stickers in packs.items():
            pack = LocalStickerPack()
            pack.title = pack_title  # type: ignore
            pack.author = author  # type: ignore

            self.add_stickers_to_pack(pack, stickers, emoji_dict)
            self.cb.put(f"Uploading pack {pack_title}")
            self.cb.put("update_bar")
            try:
                result = anyio.run(
                    UploadSignal.upload_pack,
                    pack,
                    self.opt_cred.signal_uuid,
                    self.opt_cred.signal_password,
                )
                self.cb.put((result))
                urls.append(result)

            except SignalException as e:
                self.cb.put(f"Failed to upload pack {pack_title} due to {repr(e)}")

        return urls

    @staticmethod
    def start(
        opt_output: OutputOption,
        opt_comp: CompOption,
        opt_cred: CredOption,
        cb: "Union[Queue[CbQueueItemType], Callback]",
        cb_return: CallbackReturn,
    ) -> List[str]:
        exporter = UploadSignal(opt_output, opt_comp, opt_cred, cb, cb_return)
        return exporter.upload_stickers_signal()
